#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "defs.h"

static struct input_functions *input_method = &input_evdev;
static struct output_functions *output_method = &output_uinput;

static void leave(int i)
{
	input_method->exit();
	output_method->exit();
	exit(i);
}

#define PROGRAM_DESCRIPTION \
"stick2mouse - emulates a mouse for Synaptics USB sticks\n\
\n\
Syntax:\n\
	%s [--input-method <method>] [--sensitivity <n>]\n\
\n\
Description:\n\
	--input-method <method>: libusb or evdev, default is evdev\n\
	--sensitivity <n>: n = 0..1000, default is 100\n\
\n"

static void get_params(int argc, char **argv)
{
	int i;

	for (i=1; i<argc; i++) {
		if ((!strcmp(argv[i], "--input-method")) && (i<argc-1)) {
			i++;
			if (!strcmp(argv[i], "libusb"))
				input_method = &input_libusb;
			else if (!strcmp(argv[i], "evdev"))
				input_method = &input_evdev;
			else
				goto error;
		} else if ((!strcmp(argv[i], "--sensitivity")) && (i<argc-1)) {
			if (sscanf(argv[++i], "%i", &sensitivity) != 1)
				goto error;
			if ((sensitivity < 0) || (sensitivity > 1000))
				goto error;
		} else
			goto error;
	}

	return;
error:
	printf(PROGRAM_DESCRIPTION, argv[0]);
	leave(-2);
}

int main(int argc, char **argv)
{
	int res;
	struct input_data in = { };
	struct output_data out = { };

	get_params(argc, argv);

	res = input_method->init();
	if (res)
		leave(res);
	res = output_method->init();
	if (res)
		leave(res);

	while (1) {
		res = input_method->get_data(&in);
		if (res)
			leave(res);

		stick2mouse(&in, &out);

		res = output_method->put_data(&out);
		if (res)
			leave(res);
	}

	return 0;
}
